/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.repo;

import com.google.common.cache.LoadingCache;
import org.apache.commons.lang.StringUtils;
import org.apache.http.Header;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.ProtocolException;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpHead;
import org.apache.http.protocol.HttpContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URI;
import java.util.concurrent.ExecutionException;

/**
 * Redirect strategy that blocks redirects to internal resources from remote repositories, except for internal redirects
 *
 * @author Tomer Nir
 */
public class JFrogStrictRedirectStrategy extends JFrogRedirectStrategy {
    private static final Logger log = LoggerFactory.getLogger(JFrogStrictRedirectStrategy.class);
    private final LoadingCache<String, Boolean> hostsCache;

    public JFrogStrictRedirectStrategy(LoadingCache<String, Boolean> hostsCache) {
        this.hostsCache = hostsCache;
    }

    @Override
    public boolean isRedirected(HttpRequest request, HttpResponse response, HttpContext context)
            throws ProtocolException {
        return super.isRedirected(request, response, context) && !shouldBlockRedirect(request, response);
    }

    private boolean shouldBlockRedirect(HttpRequest request, HttpResponse response) {
        try {
            URI responseRedirectLocation = getResponseRedirectLocation(response);
            if (isInetAddressFromUriLoopback(responseRedirectLocation)) {
                URI requestUri = getRequestUri(request);
                //in case of internal redirect, request uri is local and therefore should not be blocked
                return !isInetAddressFromUriLoopback(requestUri);
            }
        } catch (Exception e) {
            log.error("Unable to determine if redirect should be blocked because of the following: {}",
                    e.getMessage());
            log.debug("Unable to determine if redirect should be blocked because of the following: ", e);
        }
        return false;
    }

    private Boolean isInetAddressFromUriLoopback(URI uri) throws ExecutionException {
        if (uri == null || StringUtils.isEmpty(uri.getHost())) {
            return false;
        } else {
            return hostsCache.get(uri.getHost());
        }
    }

    private URI getResponseRedirectLocation(HttpResponse response) {
        try {
            Header locationHeader = response.getFirstHeader(HttpHeaders.LOCATION);
            String location = locationHeader.getValue();
            return URI.create(location);
        } catch (Exception e) {
            log.error("Unable to get response redirect location because of the following: {}", e.getMessage());
            log.debug("Unable to get response redirect location because of the following: ", e);
            return null;
        }
    }

    private URI getRequestUri(HttpRequest request) {
        try {
            if (request instanceof HttpGet) {
                return ((HttpGet) request).getURI();
            } else if (request instanceof HttpHead) {
                return ((HttpHead) request).getURI();
            } else {
                return null;
            }
        } catch (Exception e) {
            log.error("Unable to get request URI because of the following: {}", e.getMessage());
            log.debug("Unable to get request URI because of the following: ", e);
            return null;
        }
    }
}
