package org.artifactory.engine;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.CoreAddons;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.fs.RepoResource;
import org.artifactory.repo.Repo;
import org.artifactory.request.ArtifactoryRequest;
import org.artifactory.request.InternalRequestContext;
import org.artifactory.resource.RepoResourceInfo;
import org.artifactory.storage.binstore.service.InternalBinaryService;
import org.jfrog.storage.binstore.providers.base.ManualRedirectType;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.util.Enumeration;

import static org.jfrog.storage.binstore.ifc.model.BinaryElementHeaders.MANUAL_REDIRECT_TYPE;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.AssertJUnit.assertEquals;

/**
 * @author Shay Bagants
 */
@Test
public class DownloadServiceImplTest {

    private DownloadServiceImpl downloadService = new DownloadServiceImpl();

    @Mock
    private InternalRequestContext requestContext;
    @Mock
    private ArtifactoryRequest request;
    @Mock
    private AuthorizationService authorizationService;
    @Mock
    private DownloadRedirectHelper downloadRedirectHelper;
    @Mock
    private InternalBinaryService binaryService;
    @Mock
    private AddonsManager addonsManager;
    @Mock
    private CoreAddons aolMock;

    @BeforeMethod
    private void setup() {
        MockitoAnnotations.initMocks(this);
        when(requestContext.getRequest()).thenReturn(request);
        downloadService.authorizationService = authorizationService;
        downloadService.addonsManager = addonsManager;
        downloadService.downloadRedirectHelper = downloadRedirectHelper;
        downloadService.binaryService = binaryService;
    }

    @Test(dataProvider = "provideRequestData")
    public void testGetXrayUserName(String userAgent, String principal, String expectedResult) {
        when(authorizationService.currentUsername()).thenReturn(principal);
        when(request.getHeader("User-Agent")).thenReturn(userAgent);
        String result = downloadService.getXrayUserName(requestContext);
        Assert.assertEquals(result, expectedResult);
    }


    private void setupCdn() {
        when(downloadRedirectHelper.isRedirectEnable(any(), any())).thenReturn(true);
        when(downloadRedirectHelper.isArtifactGreaterEqualsRedirectThreshold(any())).thenReturn(true);
        when(binaryService.canCloudProviderGenerateRedirection(any())).thenReturn(true);
    }

    @Test(dataProvider = "getCdnConfigurationsData")
    public void testIsSignedUrl_differentCdnConfigurations(boolean isEplus, boolean isEdge,
                                                           boolean isAol,
                                                           boolean isAnonymous,
                                                           ManualRedirectType typeSent,
                                                           boolean hasMoreElements,
                                                           boolean expected) {
        setupCdn();

        RepoResource resource = mock(RepoResource.class);
        Repo repo = mock(Repo.class);
        LocalRepoDescriptor repoDescriptor = mock(LocalRepoDescriptor.class);
        RepoResourceInfo repoResourceInfo = mock(RepoResourceInfo.class);

        when(repoResourceInfo.getSha1()).thenReturn("cf23df2207d99a74fbe169e3eba035e633b65d94");
        when(resource.getInfo()).thenReturn(repoResourceInfo);
        when(repoDescriptor.isReal()).thenReturn(true);
        when(repo.getDescriptor()).thenReturn(repoDescriptor);

        // using params
        when(addonsManager.isEnterprisePlusInstalled()).thenReturn(isEplus);
        when(addonsManager.isEdgeLicensed()).thenReturn(isEdge);
        when(addonsManager.addonByType(CoreAddons.class)).thenReturn(aolMock);
        when(aolMock.isAol()).thenReturn(isAol);
        when(authorizationService.isAnonymous()).thenReturn(isAnonymous);
        when(request.getHeaderValues(MANUAL_REDIRECT_TYPE.getHeaderName())).thenReturn(mock(Enumeration.class));
        when(request.getHeaderValues(MANUAL_REDIRECT_TYPE.getHeaderName()).hasMoreElements()).thenReturn(hasMoreElements);
        when(request.getHeaderValues(MANUAL_REDIRECT_TYPE.getHeaderName()).nextElement()).thenReturn(typeSent.name());
        boolean isSignedUrl = downloadService.shouldSendArtifactDownloadRedirect(requestContext, repo, resource);

        assertEquals(expected, isSignedUrl);
    }

    @DataProvider
    public static Object[][] provideRequestData() {
        return new Object[][]{
                //agent, user, expectedResult
                {"xray/1.0", "token:jfxr@123", "xray"},
                {"xray/1.0", "xray", "xray"},
                {"xray/1.0", "test-user", null},
                {"jfrog-cli", "xray", null},
                {"xray/1.0", "token:jfrt@123", null},
                {"jfrog-cli", "token:jfxr@123", null},
                {"x-ray/1.0", "token:jfxr@123", null},
        };
    }

    @DataProvider
    public static Object[][] getCdnConfigurationsData() {
        return new Object[][]{
                //isEplus, isEdge, isAol, isAnonymous, manualSentType, hasMoreElements, expected
                //AOL
                {true, false, true, false, ManualRedirectType.CF, true, true},
                {false, true, true, false, ManualRedirectType.CF, true, true},
                {true, false, true, true, ManualRedirectType.CF, true, true},
                {false, true, true, true, ManualRedirectType.CF, true, true},
                {false, false, true, true, ManualRedirectType.CF, true, true},
                {false, false, true, false, ManualRedirectType.S3, true, true},
                {false, false, true, true, ManualRedirectType.S3, true, true},
                {false, false, true, false, ManualRedirectType.S3, true, true},
                {true, false, true, false, ManualRedirectType.S3, true, true},
                {false, true, true, false, ManualRedirectType.S3, true, true},
                {false, false, true, false, ManualRedirectType.S3, true, true},
                //OnPrem - isCdn cannot be true onPrem (as of 29/03/20)
                {true, false, false, false, ManualRedirectType.S3, true, true},
                {false, true, false, false, ManualRedirectType.S3, true, true},
                {false, false, false, false, ManualRedirectType.NONE, true, false},
                {false, false, false, true, ManualRedirectType.NONE, true, false},
                {false, false, false, false, ManualRedirectType.NONE, true, false},
                //Manual DIRECT request - will return direct when it's not AOL
                {true, false, false, false, ManualRedirectType.NONE, true, false},
                {true, false, false, false, ManualRedirectType.NONE, true, false},
                //Manual DIRECT request - will ignore the manual request when it's AOL
                {true, false, true, false, ManualRedirectType.NONE, true, true},
                {false, true, true, false, ManualRedirectType.NONE, true, true}
        };
    }
}