package org.artifactory.security.ssh;

import org.apache.sshd.common.Cipher;
import org.apache.sshd.common.KeyExchange;
import org.apache.sshd.common.NamedFactory;
import org.apache.sshd.common.cipher.*;
import org.apache.sshd.server.kex.DHG1;
import org.apache.sshd.server.kex.DHG14;
import org.apache.sshd.server.kex.DHGEX;
import org.apache.sshd.server.kex.DHGEX256;
import org.apache.sshd.server.kex.ECDHP256;
import org.apache.sshd.server.kex.ECDHP384;
import org.apache.sshd.server.kex.ECDHP521;
import org.artifactory.common.ArtifactoryHome;
import org.artifactory.common.ConstantValues;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

public class SshAuthServiceImplTest extends ArtifactoryHomeBoundTest {
    @Test
    public void testCipherFiltering() {
        List<NamedFactory<Cipher>> avail = Arrays.asList(
                new AES128CTR.Factory(),
                new AES192CTR.Factory(),
                new AES256CTR.Factory(),
                new ARCFOUR256.Factory(),
                new AES128CBC.Factory(),
                new TripleDESCBC.Factory(),
                new BlowfishCBC.Factory(),
                new AES192CBC.Factory(),
                new AES256CBC.Factory());
        List<NamedFactory<Cipher>> ciphers =
                SshAuthServiceImpl.filterAlgorithmByBlackList(avail, ConstantValues.cipherBlackList.getString());
        List<String> filteredCipherNames = ciphers.stream().map(NamedFactory::getName).collect(Collectors.toList());
        Assert.assertFalse(filteredCipherNames.contains(new ARCFOUR256.Factory().getName()));
    }

    @Test
    public void testKeyExchangeFilterDHG1() {
        assertKeyExchange(Collections.singletonList(new DHG1.Factory().getName()));
    }

    @Test
    public void testKeyExchangeFilterDHG1AndDHG14() {
        assertKeyExchange(Arrays.asList(new DHG1.Factory().getName(), new DHG14.Factory().getName()));
    }

    @Test
    public void testKeyExchangeFilteringDefaults() {
        assertKeyExchange(Collections.singletonList(ConstantValues.keyExchangeBlackList.getString()));
    }

    private void assertKeyExchange(List<String> excludedMethods) {
        String excludedString = null;
        if (excludedMethods != null) {
            excludedString = String.join(",", excludedMethods);
            ArtifactoryHome.get().getArtifactoryProperties()
                    .setProperty(ConstantValues.keyExchangeBlackList.getPropertyName(), excludedString);
        }
        List<NamedFactory<KeyExchange>> avail = Arrays.asList(
                new DHGEX256.Factory(),
                new DHGEX.Factory(),
                new ECDHP256.Factory(),
                new ECDHP384.Factory(),
                new ECDHP521.Factory(),
                new DHG14.Factory(),
                new DHG1.Factory());
        List<NamedFactory<KeyExchange>> keyExchange =
                SshAuthServiceImpl.filterAlgorithmByBlackList(avail, excludedString);
        List<String> expectedKeyExchangeNames = avail.stream().map(NamedFactory::getName)
                .filter(r -> excludedMethods == null || !excludedMethods.contains(r))
                .collect(Collectors.toList());
        List<String> actualKeyExchangeNames =
                keyExchange.stream().map(NamedFactory::getName).collect(Collectors.toList());
        Assert.assertEquals(actualKeyExchangeNames, expectedKeyExchangeNames);
    }

}