package org.artifactory.ui.rest.service.admin.configuration.repositories.util.validator;

import org.artifactory.addon.AddonsManager;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.common.ArtifactoryHome;
import org.artifactory.descriptor.config.CentralConfigDescriptorImpl;
import org.artifactory.descriptor.repo.RepoLayout;
import org.artifactory.descriptor.repo.RepoLayoutBuilder;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.rest.common.exception.RepoConfigException;
import org.artifactory.test.ArtifactoryHomeStub;
import org.artifactory.ui.rest.model.admin.configuration.repository.GeneralRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteCacheRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.BowerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.DockerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GenericTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.TypeSpecificConfigModel;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import javax.validation.constraints.NotNull;

import java.util.ArrayList;
import java.util.List;

import static org.artifactory.repo.config.RepoConfigDefaultValues.DEFAULT_DOCKER_REMOTE_RETRIEVAL_CACHE_PERIOD;
import static org.artifactory.repo.config.RepoConfigDefaultValues.DEFAULT_RETRIEVAL_CACHE_PERIOD;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

/**
 * @author AndreiK.
 */
@Test
public class RepoConfigValidatorTest {

    @Mock
    CentralConfigService centralConfigServiceMock;
    @Mock
    AddonsManager addonsManagerMock;
    RepoConfigValidator validator;

    @BeforeMethod
    public void setup() {
        MockitoAnnotations.initMocks(this);
        ArtifactoryHome.bind(new ArtifactoryHomeStub());
        validator = new RepoConfigValidator(centralConfigServiceMock, addonsManagerMock);
    }

    @Test(dataProvider = "provideDockerCachePeriodAndRepoType", description =
            "Actually sets the value when changed in the" +
                    " UI note the cast to DockerTypeSpecificConfigModel - this replicates the UI originated model")
    public void testRemoteRepoValidationOfRetrievalCachePeriodSecsOfCacheModel(Long cachePeriod, boolean isDockerModel,
            Long expected) {
        TypeSpecificConfigModel typeSpecificConfigModel;
        if (isDockerModel) {
            typeSpecificConfigModel = new DockerTypeSpecificConfigModel();
            ((DockerTypeSpecificConfigModel) typeSpecificConfigModel).setRetrievalCachePeriodSecs(cachePeriod);
        } else {
            typeSpecificConfigModel = new GenericTypeSpecificConfigModel();
        }
        RemoteAdvancedRepositoryConfigModel remoteAdvancedRepositoryConfigModel =
                getAdvancedRepoConfigModelAndSetRetrievalCachePeriod(cachePeriod);
        RemoteRepositoryConfigModel model = getRemoteRepositoryConfigModel(remoteAdvancedRepositoryConfigModel,
                typeSpecificConfigModel);
        RemoteCacheRepositoryConfigModel cache = model.getAdvanced().getCache();

        validator.setRetrievalCachePeriodSecsOfCacheModel(model, cache);

        assertEquals((long) cache.getRetrievalCachePeriodSecs(), (long) expected);
    }

    @Test
    public void testValidateMetadataTimeIsValidDefaultValues() throws RepoConfigException {
        RepoConfigValidator configValidator = new RepoConfigValidator(null, null);
        RemoteCacheRepositoryConfigModel cache = new RemoteCacheRepositoryConfigModel();
        configValidator.validateMetadataTimeIsValid(cache);
    }

    @Test
    public void testValidateMetadataTimeIsValidWithZeroCachePeriod() throws RepoConfigException {
        RepoConfigValidator configValidator = new RepoConfigValidator(null, null);
        RemoteCacheRepositoryConfigModel cache = new RemoteCacheRepositoryConfigModel();
        cache.setRetrievalCachePeriodSecs(0L);
        configValidator.validateMetadataTimeIsValid(cache);
    }

    @Test(expectedExceptions = RepoConfigException.class, expectedExceptionsMessageRegExp = ".*The Metadata retrieval" +
            " timeout not allowed to be bigger then Metadata Retrieval.*")
    public void testValidateMetadataTimeIsValidTimeoutBiggerThenCache() throws RepoConfigException {
        RepoConfigValidator configValidator = new RepoConfigValidator(null, null);
        RemoteCacheRepositoryConfigModel cache = new RemoteCacheRepositoryConfigModel();
        cache.setRetrievalCachePeriodSecs(1L);
        configValidator.validateMetadataTimeIsValid(cache);
    }

    @Test(expectedExceptions = RepoConfigException.class, expectedExceptionsMessageRegExp = ".*The Metadata retrieval" +
            " timeout can't be zero or negative.*")
    public void testValidateMetadataTimeIsValidWhenTimeoutIllegal() throws RepoConfigException {
        RepoConfigValidator configValidator = new RepoConfigValidator(null, null);
        RemoteCacheRepositoryConfigModel cache = new RemoteCacheRepositoryConfigModel();
        cache.setRetrievalCachePeriodSecs(1L);
        cache.setMetadataRetrievalTimeoutSecs(0L);
        configValidator.validateMetadataTimeIsValid(cache);
    }

    @Test
    public void testValidateRemoteValidFields() throws RepoConfigException {
        RemoteRepositoryConfigModel model = new RemoteRepositoryConfigModel();
        setGeneralRemoteConfig(model);
        RemoteAdvancedRepositoryConfigModel advanced = new RemoteAdvancedRepositoryConfigModel();
        RemoteCacheRepositoryConfigModel cache = new RemoteCacheRepositoryConfigModel();
        cache.setMetadataRetrievalTimeoutSecs(10L);
        advanced.setCache(cache);
        model.setAdvanced(advanced);
        setBasicRemoteConfig(model);
        BowerTypeSpecificConfigModel mock = mock(BowerTypeSpecificConfigModel.class);
        when(mock.getRepoType()).thenReturn(RepoType.Bower);
        model.setTypeSpecific(mock);
        RepoConfigValidator configValidator = createRepoConfigValidator();
        configValidator.validateRemote(model);
    }

    @Test(expectedExceptions = RepoConfigException.class, expectedExceptionsMessageRegExp = ".*The Metadata retrieval" +
            " timeout can't be zero or negative.*")
    public void testValidateRemoteNotValidMetadataTimeout() throws RepoConfigException {
        RemoteRepositoryConfigModel model = new RemoteRepositoryConfigModel();
        setGeneralRemoteConfig(model);
        RemoteAdvancedRepositoryConfigModel advanced = new RemoteAdvancedRepositoryConfigModel();
        RemoteCacheRepositoryConfigModel cache = new RemoteCacheRepositoryConfigModel();
        cache.setMetadataRetrievalTimeoutSecs(0L);
        advanced.setCache(cache);
        model.setAdvanced(advanced);
        setBasicRemoteConfig(model);
        BowerTypeSpecificConfigModel mock = mock(BowerTypeSpecificConfigModel.class);
        when(mock.getRepoType()).thenReturn(RepoType.Bower);
        model.setTypeSpecific(mock);
        RepoConfigValidator configValidator = createRepoConfigValidator();
        configValidator.validateRemote(model);
    }

    private RepoConfigValidator createRepoConfigValidator() {
        AddonsManager addonsManager = mock(AddonsManager.class);
        when(addonsManager.isEnterprisePlusInstalled()).thenReturn(true);

        CentralConfigService configService = prepareConfigService();
        return new RepoConfigValidator(configService, addonsManager);
    }

    private CentralConfigService prepareConfigService() {
        CentralConfigService configService = mock(CentralConfigService.class);
        CentralConfigDescriptorImpl centralConfigDescriptor = new CentralConfigDescriptorImpl();
        RepoLayout simpleRepoLayout = new RepoLayoutBuilder().name("simple-default")
                .artifactPathPattern("mm").distinctiveDescriptorPathPattern(false).build();
        List<RepoLayout> repoLayouts = new ArrayList<>();
        repoLayouts.add(simpleRepoLayout);
        centralConfigDescriptor.setRepoLayouts(repoLayouts);
        when(configService.getDescriptor()).thenReturn(centralConfigDescriptor);
        return configService;
    }

    private void setGeneralRemoteConfig(RemoteRepositoryConfigModel model) {
        GeneralRepositoryConfigModel general = new GeneralRepositoryConfigModel();
        general.setRepoKey("repo");
        model.setGeneral(general);
    }

    private void setBasicRemoteConfig(RemoteRepositoryConfigModel model) {
        RemoteBasicRepositoryConfigModel basic = mock(RemoteBasicRepositoryConfigModel.class);
        when(basic.getLayout()).thenReturn("simple-default");
        when(basic.getUrl()).thenReturn("http://localhost");
        model.setBasic(basic);
    }

    @DataProvider
    private Object[][] provideCachePeriodAndRepoType() {
        return new Object[][]{
                {null, true, DEFAULT_DOCKER_REMOTE_RETRIEVAL_CACHE_PERIOD},
                {null, false, DEFAULT_RETRIEVAL_CACHE_PERIOD},
                {36000L, true, 36000L},
                {36000L, false, 36000L},
        };
    }

    @NotNull
    private RemoteAdvancedRepositoryConfigModel getAdvancedRepoConfigModelAndSetRetrievalCachePeriod(Long cachePeriod) {
        RemoteAdvancedRepositoryConfigModel remoteAdvancedRepositoryConfigModel =
                new RemoteAdvancedRepositoryConfigModel();
        RemoteCacheRepositoryConfigModel remoteCacheRepositoryConfigModel = new RemoteCacheRepositoryConfigModel();
        remoteAdvancedRepositoryConfigModel.setCache(remoteCacheRepositoryConfigModel);
        remoteCacheRepositoryConfigModel.setRetrievalCachePeriodSecs(cachePeriod);
        return remoteAdvancedRepositoryConfigModel;
    }

    @NotNull
    private RemoteRepositoryConfigModel getRemoteRepositoryConfigModel(
            RemoteAdvancedRepositoryConfigModel remoteAdvancedRepositoryConfigModel,
            TypeSpecificConfigModel typeSpecificConfigModel) {
        RemoteRepositoryConfigModel remoteRepositoryConfigModel = new RemoteRepositoryConfigModel();
        remoteRepositoryConfigModel.setBasic(new RemoteBasicRepositoryConfigModel());
        remoteRepositoryConfigModel.setGeneral(new GeneralRepositoryConfigModel());
        remoteRepositoryConfigModel.setAdvanced(remoteAdvancedRepositoryConfigModel);
        remoteRepositoryConfigModel.setTypeSpecific(typeSpecificConfigModel);
        return remoteRepositoryConfigModel;
    }

    @DataProvider
    private Object[][] provideDockerCachePeriodAndRepoType() {
        return new Object[][] {
                {null, true, DEFAULT_DOCKER_REMOTE_RETRIEVAL_CACHE_PERIOD},
                {null, false, DEFAULT_RETRIEVAL_CACHE_PERIOD},
                {36000L, true, 36000L},
                {36000L, false, 36000L},
        };
    }
}